import { world, system, EquipmentSlot, EntityEquippableComponent } from "@minecraft/server";
import { damageItem } from "./dura_compont";

const CUSTOM_SHIELDS = [
    "dignified:shield",
    "elite:shield",
    "glorious:shield",
    "knight:shield",
    "majestic_shield:shield",
    "piglin_shield:shield",
    "sculk:shield",
    "wood_shield:shield"
];


const SHIELD_SOUNDS = {
    "majestic_shield:shield": "item.shield.block",
    "glorious:shield": "block.bell.hit",
    "sculk:shield": "hit.sculk_sensor",
    "piglin_shield:shield": "mob.piglin.angry",
    "wood_shield:shield": "hit.wood",
    "knight:shield": "entity.iron_golem.attack",
    "dignified:shield": "ambient.weather.lightning.impact",
    "elite:shield": "entity.ender_dragon.growl"
};



const damageMap = new Map();

function isCustomShield(item) {
    return item && CUSTOM_SHIELDS.includes(item.typeId);
}

function getShieldInHands(equip) {
    const main = equip.getEquipmentSlot("Mainhand")?.getItem();
    const off = equip.getEquipmentSlot("Offhand")?.getItem();
    if (isCustomShield(main) && !isCustomShield(off)) return { hand: "Mainhand", item: main };
    if (isCustomShield(off) && !isCustomShield(main)) return { hand: "Offhand", item: off };
    return { hand: null, item: null };
}

function getDistance3D(p1, p2) {
    return Math.hypot(p2.x - p1.x, p2.y - p1.y, p2.z - p1.z);
}

function giveEffect(entity, effectType, durationTicks, amplifier, showParticles = true) {
    const seconds = Math.ceil(durationTicks / 20);
    entity.runCommandAsync(`effect give @s ${effectType} ${seconds} ${amplifier} ${showParticles ? "" : "true"}`);
}

function applyEffect(entity, effectType, durationTicks, amplifier, showParticles = true) {
    if (entity?.addEffect) {
        entity.addEffect(effectType, durationTicks, { amplifier, showParticles });
    } else {
        giveEffect(entity, effectType, durationTicks, amplifier, showParticles);
    }
}

function playShieldSound(item, player) {
    const sound = SHIELD_SOUNDS[item.typeId] || "item.shield.block";
    player.runCommandAsync(`playsound ${sound} @s`);
}

function applyShieldEffect(item, player, damager) {
    switch (item.typeId) {
        case "majestic_shield:shield":
            applyEffect(player, "minecraft:resistance", 60, 1, true);
            break;
        case "glorious:shield":
            applyEffect(player, "minecraft:speed", 100, 1, true);
            applyEffect(player, "minecraft:regeneration", 60, 1, true);
            break;
        case "sculk:shield":
            applyEffect(damager, "minecraft:poison", 100, 2, true);
            break;
        case "piglin_shield:shield":
            const dirP = {
                x: damager.location.x - player.location.x,
                y: damager.location.y - player.location.y,
                z: damager.location.z - player.location.z
            };
            damager.applyImpulse({ x: dirP.x * 2, y: dirP.y * 1.5, z: dirP.z * 2 });
            damager.setOnFire(3, true);
            player.runCommandAsync("playsound entity.piglin.angry @s");
            break;
        case "wood_shield:shield":
            applyEffect(damager, "minecraft:slowness", 80, 1, true);
            break;
        case "knight:shield":
            applyEffect(player, "minecraft:strength", 50, 1, false);
            break;
        case "dignified:shield":
            applyEffect(player, "minecraft:regeneration", 40, 1, false);
            for (let i = 0; i < 3; i++) {
                const dx = (Math.random() - 0.5) * 4;
                const dz = (Math.random() - 0.5) * 4;
                damager.runCommandAsync(`execute at @s run summon lightning_bolt ~${dx.toFixed(2)} ~ ~${dz.toFixed(2)}`);
            }
            applyEffect(player, "minecraft:resistance", 80, 2, true);
            applyEffect(damager, "minecraft:slowness", 80, 1, true);
            break;
        case "elite:shield":
            applyEffect(player, "minecraft:strength", 60, 0, true);

            applyEffect(damager, "minecraft:weakness", 40, 0, true);
            break;
    }
}


system.runInterval(() => {
    for (const player of world.getPlayers()) {
        if (!player.isSneaking) continue;
        const equip = player.getComponent("minecraft:equippable");
        const { hand, item } = getShieldInHands(equip);
        if (!hand) continue;
        const loc = player.location;
        const view = player.getViewDirection();
        for (const e of player.dimension.getEntities()) {
            if (!e.hasComponent("minecraft:projectile")) continue;
            if (getDistance3D({ x: loc.x, y: loc.y + 0.5, z: loc.z }, e.location) < 2) {
                if (player.getGameMode() === "survival") {
                    const newItem = damageItem(item);
                    equip.setEquipment(hand, newItem);
                    playShieldSound(item, player);
                }
                applyShieldEffect(item, player, e);
                e.applyImpulse({ x: view.x * 1.5, y: view.y * 1.5, z: view.z * 1.5 });
                e.remove();
            }
        }
    }
}, 0);


world.afterEvents.entityHurt.subscribe(({ hurtEntity: player, damage }) => {
    if (player.typeId === "minecraft:player") {
        damageMap.set(player.nameTag, damage);
        system.runTimeout(() => damageMap.delete(player.nameTag), 2);
    }
});


world.afterEvents.entityHitEntity.subscribe(({ hitEntity: player, damagingEntity: damager }) => {
    if (player.typeId !== "minecraft:player" || !player.isSneaking) return;
    const equip = player.getComponent("minecraft:equippable");
    const { hand, item } = getShieldInHands(equip);
    if (!hand) return;

    const dy = player.getRotation().y - damager.getRotation().y;
    if (Math.abs(dy) <= 90) return;

    system.runTimeout(() => {
        if (player.getGameMode() === "survival") {
            const newItem = damageItem(item);
            equip.setEquipment(hand, newItem);
            playShieldSound(item, player);
        }
        const health = player.getComponent("minecraft:health");
        const blocked = damageMap.get(player.nameTag) || 0;
        if (blocked > 0) health.setCurrentValue(health.currentValue + blocked);
        if (player.hasComponent("minecraft:onFire")) player.extinguishFire();
        applyShieldEffect(item, player, damager);
    });

    const dirM = {
        x: damager.location.x - player.location.x,
        y: damager.location.y - player.location.y,
        z: damager.location.z - player.location.z
    };
    damager.applyKnockback(dirM.x, dirM.z, 1.3, 0.52);
    player.teleport(player.location, { rotation: player.getRotation() });
});
